﻿<#
.SYNOPSIS
    A script that demonstrates PowerShell integration with REST API.
.DESCRIPTION
    Authenticate and call "Get user embedded certificate(s)" REST API.
.PARAMETER uemHostAndPort
    Host and port of the UEM.
.PARAMETER tenantGuid
    Tenant guid to use for authentication.
.PARAMETER authType
    Authentication type to use.  (Local or AD).
.PARAMETER username
    Username to use for authentication.
.PARAMETER password
    Password to use for authentication.
.PARAMETER domain
    Domain to use for authentication if authType=AD.
.PARAMETER userGuid
    User guid by which the search will be performed.
.PARAMETER embeddedCertificateGuid
    Embedded certificate guid by which the search will be performed.
.EXAMPLE
    get single certificate
        .\GetEmbeddedCertificates.ps1 -uemHostAndPort server01:18084 -tenantGuid SRP00000 -authType Local -username admin -password password -domain example -userGuid c4321652-24d9-46b3-803c-e5269bded459 -embeddedCertificateGuid aff621f9-8507-42c9-ad40-7ab143f5585f
    get all certificates
        .\GetEmbeddedCertificates.ps1 -uemHostAndPort server01:18084 -tenantGuid SRP00000 -authType Local -username admin -password password -domain example -userGuid c4321652-24d9-46b3-803c-e5269bded459
#>

Param(
    [Parameter(
        Mandatory=$true,
        HelpMessage="UEM Host and port."
    )]
    [ValidateNotNullorEmpty()]
    [string]$uemHostAndPort,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Tenant guid to authenticate with."
    )]
    [ValidateNotNullorEmpty()]
    [string]$tenantGuid,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Choose (AD | Local) authType."
    )]
    [ValidateSet('AD', 'Local')]
    [string]$authType,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Username to authenticate with."
    )]
    [ValidateNotNullorEmpty()]
    [string]$username,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Password to authenticate with."
    )]
    [ValidateNotNullorEmpty()]
    [string]$password,

    [Parameter(
        Mandatory=$false,
        HelpMessage="Domain for AD authenticate type."
    )]
    [string]$domain,

    [Parameter(
        Mandatory=$true,
        HelpMessage="User guid to get user's embedded certificate."
    )]
    [ValidateNotNullorEmpty()]
    [string]$userGuid,

    [Parameter(
        Mandatory=$false,
        HelpMessage="Embedded certificate guid to get specific one."
    )]
    [ValidateNotNullorEmpty()]
    [string]$embeddedCertificateGuid
)

$Global:authString = $null;
$Global:uemBaseUrl = New-Object -TypeName System.Text.StringBuilder;

[System.Net.ServicePointManager]::SecurityProtocol = [System.Net.SecurityProtocolType]::Tls12;

<#
    * Generate authorization header required to perform the REST call.
    *
    * @return
    *       True if successful, false otherwise.
#>
function setup()
{
    $methodName = "setup()";
    enterMethod($methodName);
    $provider = $null;

    $Global:uemBaseUrl.Append("https://");
    $Global:uemBaseUrl.Append($uemHostAndPort);
    $Global:uemBaseUrl.Append("/");
    $Global:uemBaseUrl.Append($tenantGuid);
    $Global:uemBaseUrl.Append("/api/v1");

    # Create and set the headers for the REST request
    $headers = @{"Content-Type" = "application/vnd.blackberry.authorizationrequest-v1+json"};

    # Create body for REST call
    $body = @{};
    $body.add("provider",$authType);
    $body.add("username", $username);
    $body.add("password", [Convert]::ToBase64String([System.Text.Encoding]::UTF8.GetBytes($password)));

    if($authType -eq "AD")
    {
        if(-not($domain))
        {
            throw [System.ArgumentNullException]"The domain argument is required when authType=AD";
        }
        $body.add("domain", $domain);
    }

    $request = New-Object -TypeName System.Text.StringBuilder;
    $request.Append($Global:uemBaseUrl);
    $request.Append("/util/authorization");

    # Invoke RESTMethod to call authorization route to generate the authorization headers
    try
    {
        $response = Invoke-RestMethod -Method POST -Headers $headers -Uri $request.ToString() -Body ($body | ConvertTo-Json);
        $Global:authString = $response
    }
    catch
    {
        handleError $_;
    }

    Write-Host([string]::Format("Authorization header string generated: {0}", $Global:authString));
    Write-Host("Setup Complete...");

    exitMethod($methodName);

    return $true;
}

<#
    * Logging for method entry.
    *
    * @param methodName
    *       Name of the method to log
#>
function enterMethod($methodName)
{
    Write-Host([string]::Format("Entering {0}...", $methodName));
}

<#
    * Logging for method exit.
    *
    * @param methodName
    *       Name of the method to log.
#>
function exitMethod($methodName)
{
    Write-Host([string]::Format("Exiting {0}...", $methodName));
}

<#
    *Obtaining user embedded certificate or all certificates if certificate guid is null or empty
    *
    * @param userGuid
    *       Search certificate by specific user guid
    * @param embeddedCertificateGuid
    *        Search certificate by specific certificate guid, can be null or empty
#>
function getEmbeddedCertificates()
{
    $methodName = "getEmbeddedCertificate()";
    enterMethod($methodName);

    $request = New-Object -TypeName System.Text.StringBuilder;
    $response = $null;

    # Create and set the headers for the REST request
    $headers = @{}
    $headers.add("Authorization", $Global:authString);
    $headers.add("Content-Type", "application/vnd.blackberry.users-v1+json");

    $request.Append($Global:uemBaseUrl);
    $request.Append("/users/");
    $request.Append($userGuid);
    $request.Append("/embeddedCertificates");

    if(![string]::IsNullOrEmpty($embeddedCertificateGuid))
    {
        $request.Append("/").Append($embeddedCertificateGuid);
    }

    # Invoke RESTMethod to call API to obtain user embedded certificate in the system
    try
    {
        $response = Invoke-RestMethod -Method GET -Headers $headers -Uri $request.ToString();
    }
    catch
    {
         handleError $_;
    }

    exitMethod($methodName);
    return $response;
}

<#
    * Handle any errors that occur when performing the REST call.
    *
    * @param error
    *       The group create context.
#>
function handleError($error)
{
    Write-Error ("The command cound not be completed.  `r`nReason: " + $error.exception.message + " `r`nDetails: "+ $error.errordetails);
    exit(1);
}

function main()
{
    $methodName = "main()";
    $response = $null;
    enterMethod($methodName);
    try
    {
        if (setup)
        {
            $response = getEmbeddedCertificates;
        }
    }
    catch
    {
        handleError $_;
    }
    Write-Host($response | Select-Object -Last 1 | ConvertTo-Json -Depth 4);

    exitMethod($methodName);
    Write-Host("Complete!");
    exit(0);
}

# Call main function.
main;